"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const jsx_runtime_1 = require("react/jsx-runtime");
const react_1 = require("react");
const client_1 = require("react-dom/client");
function reactToCustomElement(Component, { strictMode = true, shadowMode, properties = {}, attributes = [], }) {
    const observedAttributes = [];
    const props = [];
    for (const key of Object.keys(properties)) {
        props.push(key);
    }
    for (const attributeName of attributes) {
        observedAttributes.push(attributeName);
    }
    // https://react.dev/blog/2024/04/25/react-19#support-for-custom-elements
    // https://github.com/facebook/react/blob/9d4fba078812de0363fe9514b943650fa479e8af/packages/react-dom-bindings/src/client/DOMPropertyOperations.js#L189-L231
    class CustomElement extends HTMLElement {
        static observedAttributes = observedAttributes;
        container;
        root;
        connected = false;
        componentProps = {};
        componentAttributes = {};
        constructor() {
            super();
            this.container = shadowMode
                ? this.attachShadow({ mode: shadowMode })
                : this;
            this.root = (0, client_1.createRoot)(this.container);
            // React detects if the node has the corresponding properties otherwise
            // props gets set as attributes.
            for (const propertyName of props) {
                Object.defineProperty(this, propertyName, {
                    enumerable: true,
                    get() {
                        return this.componentProps[propertyName];
                    },
                    set(value) {
                        this.componentProps[propertyName] = value;
                        this.render();
                    },
                });
            }
        }
        connectedCallback() {
            this.connected = true;
            this.render();
        }
        disconnectedCallback() {
            this.connected = false;
            this.root.unmount();
        }
        attributeChangedCallback(attribute, _, value) {
            this.componentAttributes[attribute] = value;
            this.render();
        }
        render() {
            if (!this.connected) {
                return;
            }
            const Wrapper = strictMode ? react_1.StrictMode : react_1.Fragment;
            this.root.render((0, jsx_runtime_1.jsx)(Wrapper, { children: (0, jsx_runtime_1.jsx)(Component, { ...this.componentAttributes, ...this.componentProps }) }));
        }
    }
    return CustomElement;
}
exports.default = reactToCustomElement;
